/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import { useAppDispatch, useQuerybuilderGraph, useQuerybuilderHash } from '@graphpolaris/shared/lib/data-access';
import { useCallback, useEffect, useMemo, useState } from 'react';
import { Handle, HandleType, Position } from 'reactflow';
import { LogicNodeAttributes, SchemaReactflowLogicNode, toHandleId } from '../../../model';
import { InputNode, InputNodeTypeTypes } from '../../../model/logic/general';
import { styleHandleMap } from '../../utils';
import styles from './logicpill.module.scss';
import { setQuerybuilderGraphology, toQuerybuilderGraphology } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';

/**
 * Component to render an entity flow element
 * @param param0 Data of the flow element.
 */
export default function LogicPill(node: SchemaReactflowLogicNode) {
  const dispatch = useAppDispatch();
  const data = node.data;
  const logic = data.logic;
  const output = data.logic.output;

  const graph = useQuerybuilderGraph();
  const graphologyHash = useQuerybuilderHash();
  const graphologyGraph = useMemo(() => toQuerybuilderGraphology(graph), [graph]);
  const connectionsToLeft = useMemo(() => graph.edges.filter((edge) => edge.target === node.id), [graph]);
  const connectionsToRight = useMemo(() => graph.edges.filter((edge) => edge.source === node.id), [graph]);
  const graphologyNodeAttributes = useMemo<LogicNodeAttributes | undefined>(
    () => (graphologyGraph.hasNode(node.id) ? { ...(graphologyGraph.getNodeAttributes(node.id) as LogicNodeAttributes) } : undefined),
    [node.id]
  );
  const [localInputCache, setLocalInputCache] = useState<Record<string, InputNodeTypeTypes>>({ ...graphologyNodeAttributes?.inputs });

  if (!data.id) throw new Error('LogicPill: data.id is undefined');
  const defaultHandleData = {
    nodeId: data.id,
    nodeName: data.name,
    nodeType: data.type,
  };

  const onInputUpdated = (value: string, input: InputNode, idx: number) => {
    let logicNode = { ...graphologyNodeAttributes };
    if (!logicNode) throw new Error('LogicPill: logicNode is undefined');

    let logicNodeInputs = { ...logicNode.inputs };
    if (data.inputs[input.name] !== value) {
      logicNodeInputs[input.name] = value;
      logicNode.inputs = logicNodeInputs;
      graphologyGraph.setNodeAttribute<any>(node.id, 'inputs', logicNodeInputs); // FIXME: I'm not sure why TS requires <any> to work here
      dispatch(setQuerybuilderGraphology(graphologyGraph));
      // console.log('updated input', input.name, 'from', data.inputs[input.name], 'to', value);
    }
  };

  // const createLeftHandles = useCallback(
  //   (sideInputs: InputNode[]) => {
  //     return sideInputs.filter((input, i) => {
  //       return !connectionsToLeft.some(
  //         (edge) => edge?.attributes?.targetHandleData.nodeId === data.id && edge?.attributes?.targetHandleData.attributeName === input.name
  //       );
  //     }).length;
  //   },
  //   [node]
  // );
  // const leftInputsNumber = createLeftHandles(node.data.logic.inputs);

  return (
    <div className={styles.logic + ' w-fit h-min-[3rem]'}>
      <div className="h-fit">
        {
          <div className="m-1 mx-2 text-left">
            {connectionsToLeft.map((e) => e?.attributes?.sourceHandleData.attributeName)}.{output.name}
          </div>
        }
        {node.data.logic.inputs.map((input, i) => {
          let inputTextBox = null;
          if (
            !connectionsToLeft.some(
              (edge) =>
                edge?.attributes?.targetHandleData.nodeId === data.id && edge?.attributes?.targetHandleData.attributeName === input.name
            )
          ) {
            inputTextBox = (
              <input
                className="px-0.5 m-2 mt-0 h-5 border-logic-600 rounded-sm border-[1px]"
                style={{ width: input.type === 'string' ? '9rem' : '4rem' }}
                placeholder="empty"
                value={localInputCache?.[input.name] as string}
                onChange={(e) => {
                  setLocalInputCache({ ...localInputCache, [input.name]: e.target.value });
                }}
                onKeyDown={(e) => {
                  if (e.key === 'Enter') onInputUpdated((e.target as HTMLInputElement).value, input, i);
                }}
                onBlur={(e) => onInputUpdated(e.target.value, input, i)}
              />
            );
          }
          return (
            <div key={i}>
              <div className="w-full flex">{inputTextBox}</div>
              <Handle
                type={'target'}
                position={Position.Left}
                id={toHandleId({ ...defaultHandleData, attributeName: input.name, attributeType: input.type })}
                key={input.name + input.type}
                style={{ top: `${((i + 0.8) / (node.data.logic.inputs.length + 0.6)) * 100}%` }}
                className={styleHandleMap[input.type] + ''}
              ></Handle>
            </div>
          );
        })}
        {!!node.data.logic.output && (
          <Handle
            type={'source'}
            position={Position.Right}
            id={toHandleId({ ...defaultHandleData, attributeName: output.name, attributeType: output.type })}
            className={styleHandleMap?.[output.type]}
          ></Handle>
        )}
      </div>
    </div>
  );
}
